@echo on
@SETLOCAL
@REM  First check to make sure you are not running this 
@REM  from the \ASA9\WIN32 or WIN directory.
if exist dbeng9.exe goto WrongDirectory

@SET COLLATION=1252LATIN1
@if NOT DEFINED ISQL SET ISQL=dbisql -nogui -onerror exit
@SET PAUSE_BETWEEN_STEPS=

if NOT DEFINED DBMLSYNC_SWITCHES SET DBMLSYNC_SWITCHES=-ot remote.txt -v
SET CONS_DSN=cons
SET PROXY_DSN=cons_proxy
SET REMOTE_DSN=remote
SET SQL_CUSTOMER="SELECT db_property('name') AS which_database, * FROM Customer"
SET SQL_CUSTOMER_DELETED="SELECT db_property('name') AS which_database, *, now(*) AS CurrentTime FROM Customer_deleted"

@if %2. NEQ . GOTO ONE_SYNC

@if %1. EQU all. SET DBMLSYNC_SWITCHES=%DBMLSYNC_SWITCHES% -k
@if %1. EQU . SET PAUSE_BETWEEN_STEPS=1
@if %1. NEQ . GOTO %1

:ALL
@echo.
@echo.************************************************************
@echo.Stop both the MobiLink server and the database engine
@echo.Delete databases from old runs
@echo.************************************************************
@echo.
@echo on
dbmlstop -w 
dbstop -y cons
@echo off
if exist consdb attrib -r consdb\*.*
if exist remotedb attrib -r remotedb\*.*
if exist backup attrib -r backup\*.*

@echo.
@echo.************************************************************
@echo.Remove the DSN created for the course
@echo.************************************************************
@echo.
@echo on
dbdsn -d %CONS_DSN% -y
dbdsn -d %PROXY_DSN% -y
dbdsn -d %REMOTE_DSN% -y
@echo off

@echo.
@echo.************************************************************
@echo.Remove all left over directories
@echo.************************************************************
@echo.
if exist consdb (echo y|del /Q consdb) 
if exist remotedb (echo y|del /Q remotedb) 
if exist unload (echo y|del /Q unload)
if exist backup (echo y|del /Q backup)
if exist consdb rmdir consdb
if exist remotedb rmdir remotedb
if exist backup rmdir backup
if exist unload rmdir unload
if exist *.mle del *.mle
if exist *.txt del *.txt
if exist 1.sql del 1.sql
if exist 2.sql del 2.sql

@echo.
@echo.************************************************************
@echo.  All files are cleaned up now
@echo.  The remaining files are needed to setup synchronization.
@echo.  Use CTRL-C to stop ML.BAT
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause

REM  Create a temporary directory and store an empty DB.
REM  This is useful if you are running ML.BAT over and over.
if not exist tempDB md tempDB
@echo on
if not exist tempDB\cons.db dbinit -z %COLLATION% tempDB\cons.db
@echo off

for /f %%i in ('cd') do @set curr_dir=%%i
cd ..
for /f %%i in ('cd') do @set db_dir=%%i
cd %curr_dir%


@echo.
@echo.************************************************************
@echo.Recreate all the ODBC DSNs 
@echo.************************************************************
@echo.
dbdsn -w %CONS_DSN% -v -c uid=dba;pwd=sql;eng=cons;dbf=%curr_dir%\consdb\cons.db;links=shmem -y
dbdsn -w %PROXY_DSN% -v -c uid=dba;pwd=sql;eng=cons;dbf=%curr_dir%\consdb\cons_proxy.db;links=shmem -y
dbdsn -w %REMOTE_DSN% -v -c uid=dba;pwd=sql;eng=cons;dbf=%curr_dir%\remotedb\remote.db;links=shmem -y

mkdir consdb 
mkdir remotedb 
mkdir unload

@echo.
@echo.************************************************************
@echo.Copy blank databases (faster than creating)
@echo.************************************************************
@echo.
copy tempDB\cons.db consdb\cons.db
copy tempDB\cons.db consdb\cons_proxy.db
copy tempDB\cons.db remotedb\remote.db
@echo on
dblog -t cons_proxy.log consdb\cons_proxy.db
dblog -t remote.log remotedb\remote.db
@echo off

@echo.
@echo.************************************************************
@echo.Start the database and issue SQL commands
@echo.************************************************************
@echo.
@echo on
dbspawn -f dbeng9 -x tcpip consdb\cons.db consdb\cons_proxy.db
@echo off

:START_ML
@echo.
@echo.************************************************************
@echo.Start the MobiLink server
@echo.************************************************************
@echo.
@echo on
start /min dbmlsrv9 -zu+ -za -vcrs -x tcpip{port=2439} -c "dsn=%CONS_DSN%" -ot ml.txt 
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS
@if %1. EQU START_ML. goto DONE
rem start /min dbmlsrv9 -dl -vcrs -x tcpip{port=2439;security=certicom_tls{certificate=serv1.crt;certificate_password=password}} -c "dsn=%CONS_DSN%" -ot ml.txt

:STEP1
@echo.
@echo.************************************************************
@echo.Step 1 - Create consolidated database 
@echo.       - Create proxy database 
@echo.
@echo.options.sql       - Some common database options for synchronization
@echo.tables_common.sql - Tables used by the remote and proxy database
@echo.initial_data.sql  - Initially populates the consolidated
@echo.procedures.sql    - Procedures used by proxy database
@echo.proxy_setup.sql   - Creates a remote server to the proxy database
@echo.cons_proxy_references.sql      
@echo.                  - Defines the interface between the consolidated
@echo.                    and the proxy database
@echo.sync_scripts.sql  - Synchronization scripts used by MobiLink
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
%ISQL% -c "dsn=%PROXY_DSN%" read tables_common.sql;read initial_data.sql; read procedures.sql; 
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS
@echo on
%ISQL% -c "dsn=%CONS_DSN%" read options.sql; read proxy_setup.sql [%PROXY_DSN%] [DBA] [SQL];
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS
@echo on
%ISQL% -c "dsn=%CONS_DSN%" read cons_proxy_references.sql; read sync_scripts.sql;
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP2
@echo.
@echo.************************************************************
@echo.Step 2 - Create remote databases 
@echo.
@echo.options.sql       - Some common database options for synchronization
@echo.tables_common.sql - Tables used by the remote and proxy database
@echo.remote_sub.sql    - Creates a publication, sync user, sync subscription
@echo.                    takes parameters to be reused.
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
%ISQL% -c "dsn=%REMOTE_DSN%" read options.sql; read tables_common.sql; 
@echo off
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
%ISQL% -c "dsn=%REMOTE_DSN%" read remote_sub.sql [r1] [tcpip] [localhost] [2439]
@echo off
@if not %errorlevel% EQU 0 goto DONE

:STEP3
@echo.
@echo.************************************************************
@echo.Step 3 - Run dbmlsync for the first time to download
@echo.         initial data.
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
dbmlsync -c "dsn=%REMOTE_DSN%" %DBMLSYNC_SWITCHES%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

@echo on
%ISQL% -c "dsn=%CONS_DSN%" %SQL_CUSTOMER%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP4
@echo.
@echo.************************************************************
@echo.Step 4 - Delete customer, 101, at the remote.
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
%ISQL% -c "dsn=%REMOTE_DSN%" "DELETE FROM Customer WHERE cust_id = 101; COMMIT;"
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP5
@echo.
@echo.************************************************************
@echo.Step 5 - Run dbmlsync 
@echo.         Check the Customer table to ensure it was successfully
@echo.         applied.  
@echo.         Customer, 101, should NOT exist at CONSOLIDATED
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
dbmlsync -c "dsn=%REMOTE_DSN%" %DBMLSYNC_SWITCHES%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

@echo on
%ISQL% -c "dsn=%CONS_DSN%" %SQL_CUSTOMER%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP6
@echo.
@echo.************************************************************
@echo.Step 6 - Insert a new customer, 105, at the remote.
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
%ISQL% -c "dsn=%REMOTE_DSN%" "INSERT INTO Customer(cust_id, first_name, last_name) VALUES(105, 'Foghorn', 'Leghorn' ); COMMIT;"
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP7
@echo.
@echo.************************************************************
@echo.Step 7 - Ensure the new customer is displayed.
@echo.         This will perform a standard insert into the 
@echo.         proxy table.
@echo.************************************************************
@echo.
@echo on
dbmlsync -c "dsn=%REMOTE_DSN%" %DBMLSYNC_SWITCHES%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

@echo on
%ISQL% -c "dsn=%CONS_DSN%" %SQL_CUSTOMER%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP6
@echo.
@echo.************************************************************
@echo.Step 7 - Update customer, 105, at the remote.
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
%ISQL% -c "dsn=%REMOTE_DSN%" "UPDATE Customer SET first_name = 'Elmer', last_name = 'Fud' WHERE cust_id = 102; COMMIT;"
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

:STEP8
@echo.
@echo.************************************************************
@echo.Step 8 - Ensure the update is applied successfully.
@echo.         The upload_update event is calling a REMOTE
@echo.         PROCEDURE CALL to update the data in the proxy 
@echo.         database.  This was done just for demonstration
@echo.         purposes.
@echo.************************************************************
@echo.
@echo on
dbmlsync -c "dsn=%REMOTE_DSN%" %DBMLSYNC_SWITCHES%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

@echo on
%ISQL% -c "dsn=%CONS_DSN%" %SQL_CUSTOMER%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

goto DONE

:DBMLSYNC
@echo.
@echo.************************************************************
@echo.Run dbmlsync 
@echo.
@echo.************************************************************
@echo.
@IF DEFINED PAUSE_BETWEEN_STEPS pause
@echo on
dbmlsync -c "dsn=%REMOTE_DSN%" %DBMLSYNC_SWITCHES%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

@echo on
%ISQL% -c "dsn=%CONS_DSN%" %SQL_ENG_MSGS%
@echo off
@if NOT %errorlevel% EQU 0 goto ERRORS

goto DONE

:DONE
@echo.
@echo.************************************************************
@echo.All done tests.
@echo.************************************************************
@echo.

goto END

:ERRORS
@echo.
@echo.************************************************************
@echo.ERROR
@echo.Tests did not complete properly.
@echo.************************************************************
@echo.

goto END

:WrongDirectory
@echo off
cls
@echo.
@echo.************************************************************
@echo.   STOP!!
@echo.You need to create a new/empty directory and copy 
@echo.ML.BAT and the SQL files into that directory.
@echo.************************************************************
@echo.
goto END

:END
@ENDLOCAL
